unit acmwave;

interface

uses sysutils, messages, msacm, mmsystem, dialogs, windows, forms;

type
 TOnData = procedure(data:pointer; size:longint) of object;
 TACMWaveOut = class
    private
       HWaveOut1: PHWaveOut;
       closed: boolean;
    public
       function Open(format:PWaveFormatEx): integer;
       function isClosed: boolean;
       procedure Playback(data:pointer;size:longint);
       procedure WaveOutCallback(var msg:TMessage);
       procedure Close;
       constructor Create;
    end;

 TACMWaveIn = class
    private
       closed: boolean;
       sizebuf: integer;
       HWaveIn1: PHWaveIn;
    public
       userCallback: TOnData;
       function isClosed: boolean;
       function Open(format:PWaveFormatEx; h: HWND): integer;
       procedure Close;
       procedure WaveInCallback (var msg:TMessage);
       constructor Create;
    end;

implementation

constructor TACMWaveOut.Create;
begin
 closed:=true;
 HWaveOut1:=nil;
end;

function TACMWaveOut.isClosed: boolean;
begin
 result:=closed;
end;

function TACMWaveOut.Open(format:PWaveFormatEx): integer;
var
 waveformat: PWaveFormatEx;
 maxsizeformat,i: integer;
begin
 i:=0;
 if (format<>nil) and (HWaveOut1=nil) then begin
    acmMetrics(0, ACM_METRIC_MAX_SIZE_FORMAT,MaxSizeFormat);
    getmem(WaveFormat, MaxSizeFormat);
    move(format^,waveformat^,maxsizeformat);
    HWaveOut1:=new(PHWaveOut);
    i:=WaveOutOpen(HWaveOut1,0,waveformat,Application.Handle,0,CALLBACK_WINDOW or WAVE_MAPPED);
    if i<>0 then begin
       result:=i;
       exit;
       end;
    closed:=false;
    end;
 result:=i;
end;


procedure TACMWaveOut.Playback(data:pointer;size:longint);
var
 Header: PWaveHdr;
 memblock: pointer;
 i: integer;
begin
 if HWaveOut1<>nil then begin
    header:=new(PWaveHdr);
    getmem(memblock,size);
    move(data^,memBlock^,size);
    header.lpdata:=memBlock;
    header.dwbufferlength:=size;
    header.dwbytesrecorded:=size;
    header.dwUser:=0;
    header.dwflags:=0;
    header.dwloops:=0;
    i:=WaveOutPrepareHeader(HWaveOut1^,header,sizeof(TWaveHdr));
    if i<> 0 then showmessage('WaveOutPrepareHeader error');
    i:=WaveOutWrite(HWaveOut1^,header,sizeof(TWaveHdr));
    if i<> 0 then showmessage('WaveOutWrite error');
    end;
end;

procedure TACMWaveOut.WaveOutCallback(var msg:TMessage);
var
 header: PWaveHdr;
 i: integer;
begin
 header:=PWaveHdr(msg.lParam);
 if closed=false then begin
    i:=WaveOutUnPrepareHeader(HWaveOut1^,header,sizeof(TWaveHdr));
    if i<>0 then showmessage('Error in WaveOutUnPrepareHeader');
    end;
 dispose(header^.lpData);
 dispose(header);
end;

procedure TACMWaveOut.Close;
begin
 if HWaveOut1<>nil then begin
    closed:=true;
    WaveOutReset(HWaveOut1^);
    WaveOutClose(HWaveOut1^);
    HWaveOut1:=nil;
    end;
end;

procedure TACMWaveIn.WaveInCallback(var msg:TMessage);
var
 Header:PWaveHdr;
 i,bytesrecorded:integer;
 data:PChar;
begin
 Header:=PWaveHdr(msg.lparam);
 if closed=false then begin
    i:=waveInUnPrepareHeader(HWaveIn1^,Header,sizeof(TWavehdr));
    if i<>0 then showmessage('Error in waveInUnPrepare');
    bytesrecorded:=header.dwbytesrecorded;
    getmem(data,bytesrecorded);
    move(header.lpdata^,data^,bytesrecorded);
    userCallback(data,bytesrecorded);
    Freemem(data);
    header.dwbufferlength:=sizebuf;
    header.dwbytesrecorded:=0;
    header.dwUser:=0;
    header.dwflags:=0;
    header.dwloops:=0;
    /* SEGV here: closed=true? HWaveIn=nil */
    i:=waveInPrepareHeader(HWaveIn1^,Header,sizeof(TWavehdr));
    if i<>0 then showmessage('Error in waveInPrepareHeader');
    i:=waveInAddBuffer(HWaveIn1^,Header,sizeof(TWaveHdr));
    if i<>0 then showmessage('Error in waveInAddBuffer');
    end
 else begin
    dispose(header.lpdata);
    dispose(header)
    end;
end;

constructor TACMWaveIn.Create;
begin
 HWaveIn1:=nil;
 closed:=true;
end;

function TACMWaveIn.isClosed: boolean;
begin
 result:=closed;
end;

function TACMWaveIn.Open(format:PWaveFormatEx; h: HWND): integer;
var
 WaveFormat: PWaveFormatEx;
 Header: PWaveHdr;
 memBlock: PChar;
 i,j,maxsizeformat: integer;
begin
 i:=0;
 if (HwaveIn1=nil) and (format<>nil) then begin
    acmMetrics(0,ACM_METRIC_MAX_SIZE_FORMAT,MaxSizeFormat);
    getmem(WaveFormat, MaxSizeFormat);
    move(format^,waveformat^,maxsizeformat);
    sizebuf:=format.nAvgBytesPerSec;
    HWaveIn1:=new(PHWaveIn);
    //i:=WaveInOpen(HWaveIn1,0,waveformat,handle,0,CALLBACK_WINDOW or WAVE_MAPPED);
    i:=WaveInOpen(HWaveIn1,WAVE_MAPPER,waveformat,h,0,CALLBACK_WINDOW);

    if i<>0 then begin
       result:=i;
       exit;
       end;
    closed:=false;
    for j:= 1 to 3 do begin
       header:=new(PWaveHdr);
       getmem(memblock,sizebuf);
       header.lpdata:=memBlock;
       header.dwbufferlength:=sizebuf;
       header.dwbytesrecorded:=0;
       header.dwUser:=0;
       header.dwflags:=0;
       header.dwloops:=0;
       i:=waveInPrepareHeader(HWaveIn1^,Header,sizeof(TWavehdr));
       if i<>0 then showmessage('In Prepare error');
       i:=waveInAddBuffer(HWaveIn1^,Header,sizeof(TWaveHdr));
       if i<>0 then showmessage('Add buffer error');
       end;
    i:=waveInStart(HWaveIn1^);
    if i<>0 then showmessage('Error in waveInStart');
 end;
 result:=i;
end;

procedure TACMWaveIn.Close;
begin
 if HWaveIn1<>nil then begin
    closed:=true;
    WaveInReset(HWaveIn1^);
    WaveInClose(HWaveIn1^);
    dispose(HWaveIn1);
    HWaveIn1:=nil;
    end;
end;

end.
