/***************************************************************************
                          ImageBox.cpp  -  description
                             -------------------
    begin                : ven mag 7 2004
    copyright            : (C) 2004 by Daniele Lavatelli
    email                : ldanilinux@interfree.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "Servers.h"
#include "ServerDialog.h" 
#include "ImageBox.h"
#include "Net.h"
#include <qcursor.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <qnamespace.h>
#include <pthread.h>
#include <qapplication.h>
#include <qpainter.h>

#define EVENT_DRAWFRAME            0
#define EVENT_REDRAWBACKGROUND     1
#define EVENT_REDRAWFRAME          2

QColor NOTSEL_NOERRORS(0,128,0);
QColor NOTSEL_WARNING(128,128,0);
QColor NOTSEL_CRITICAL(128,0,0);
QColor SEL_NOERRORS(0,255,0);
QColor SEL_WARNING(255,255,0);
QColor SEL_CRITICAL(255,0,0);

static const char rcsid[]="$Id: ImageBox.cpp,v 1.2 2004/07/03 13:17:09 santinod Exp $";

static QWidget *gParent;

ImageBox::ImageBox(QWidget *parent, const char *name, int x, int y) : QWidget(parent,name,WRepaintNoErase)
{
 setMinimumSize(0,0);
 //Store coordinates in MainWindow's grid
 mX = x;
 mY = y;
 //Reset error state and initialize server to "disconnected"
 mErrorStatus = ERR_NOERRORS;
 mDrawActive = false;
 sprintf(mStatusMessage,"Disconnected.");
 sprintf(mServerName,"Not connected");
 sprintf(mServerPort,"Default");
 mSelected = false;
 mJpegFrame.jpeg_buffer=0;

 pthread_mutex_init(&mBusy,NULL);
 mThId = 0;
 
 //Create menu
 mpMenu = new QPopupMenu(this,"menu");
 connect_id = mpMenu->insertItem("Connect",this,SLOT( Connect() ));
 reconnect_id = mpMenu->insertItem("Reconnect",this,SLOT( Reconnect() ));
 disconnect_id = mpMenu->insertItem("Disconnect",this,SLOT( Disconnect() ));
 inspect_id = mpMenu->insertItem("Inspect this",this,SLOT( Inspect() ));
 
 normal_id = mpMenu->insertItem("Original size",this,SLOT( ShowNormal() ));
 stretch_id = mpMenu->insertItem("Stretch",this,SLOT( ShowStretch() ));
 fit_id = mpMenu->insertItem("Fit - Keep ratio",this,SLOT( ShowFit() ));
 mpMenu->insertSeparator(3);
 mpMenu->insertSeparator(5);

 mpMenu->setItemChecked(normal_id,true);
 mpMenu->setItemEnabled(reconnect_id,false);
 mpMenu->setItemEnabled(disconnect_id,false);
 mpMenu->setItemEnabled(inspect_id,false);

 mpMenu->setItemEnabled(normal_id,false);
 mpMenu->setItemEnabled(fit_id,false);
 mpMenu->setItemEnabled(stretch_id,false);

 mpBox = new QLabel(this,"viewbox",WRepaintNoErase);

 strcpy(mServerName,"Not connected");
 
 setPaletteBackgroundColor( NOTSEL_NOERRORS );
 erase();
 
 gParent = parent;

#ifndef EXPERIMENTAL
 inspect_id = 99;
#endif
}

ImageBox::~ImageBox()
{
 //Force disconnection...
 if (mThId) pthread_cancel(mThId);
 //Delete controls...
 delete mpMenu;
 delete mpBox;
 //Delete frame buffer...
 if(mJpegFrame.jpeg_buffer) free(mJpegFrame.jpeg_buffer);
}

void ImageBox::mousePressEvent(QMouseEvent *e)
{
 //MainWindow must adjust selection...
 emit Clicked(mX,mY);
 //Execute menu at current mouse position
 if (e->button() == RightButton) mpMenu->exec(QCursor::pos()); 
 gParent->repaint(); 
}

void ImageBox::ShowStretch()
{//Change menu check status
 mpMenu->setItemChecked(stretch_id,true);
 mpMenu->setItemChecked(fit_id,false);
 mpMenu->setItemChecked(normal_id,false);
 //Update ImageBox
 mpBox->setGeometry(1,1,width()-2,height()-2);
// erase();
 repaint();
}

void ImageBox::ShowFit()
{int w,h;
 float ratio;
 //Change menu check status
 mpMenu->setItemChecked(stretch_id,false);
 mpMenu->setItemChecked(fit_id,true);
 mpMenu->setItemChecked(normal_id,false);
 //Update ImageBox
 ratio = (float)mImage.width()/(float)mImage.height();
 if (ratio > (float)width()/(float)height())
 {     
  w = width();
  h = (int)((float)width()/ratio);
  mpBox->setGeometry( 1 , (int)(height() - h)/2 , w-2 , h);
 }
 else
 {
  h = height();
  w = (int)((float)height()*ratio);
  mpBox->setGeometry( (int)(width() - w)/2 , 1 , w , h-2);
 }
 repaint(); 
}

void ImageBox::ShowNormal()
{int w,h;
 //Change menu check status
 mpMenu->setItemChecked(stretch_id,false);
 mpMenu->setItemChecked(fit_id,false);
 mpMenu->setItemChecked(normal_id,true);
 //Update ImageBox
 if (mImage.width() < width()-2) w = mImage.width(); else w = width()-2;
 if (mImage.height() < height()-2) h = mImage.height(); else h = height()-2;
 mpBox->setGeometry(1,1,w,h);
 erase();
// repaint();
}

void ImageBox::Connect()
{
 ServerDialog *server_dialog = new ServerDialog(this,"serverdialog",true,0);

 if (server_dialog->exec()==QDialog::Accepted)
 {
  strcpy(mServerName,server_dialog->GetServerName());
  strcpy(mServerPort,server_dialog->GetServerPort());

  //Prepare buffer...
  if(mJpegFrame.jpeg_buffer==0) {
   if( (mJpegFrame.jpeg_buffer = (unsigned char *)malloc(60000)) == 0)
   {
    QMessageBox::critical(this,"Palantir connection","Errors allocating jpeg buffer",QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
    return;
   }
   mJpegFrame.jpeg_buffer_length=60000;
  }

  //Delete temp_frame...
  temp_frame.reset();
  mpBox->hide();
  erase();
  mpBox->show();
        
  //Start draw loop...
  //mDrawActive = true; /* moved into main loop -- DS */
  pthread_create(&mThId,NULL,StartThread,this);

 }
 delete server_dialog;
}

void ImageBox::Reconnect()
{
 //Prepare buffer...
 if(mJpegFrame.jpeg_buffer==0) {
  if( (mJpegFrame.jpeg_buffer = (unsigned char *)malloc(60000)) == 0)
  {
   QMessageBox::critical(this,"Palantir connection","Errors allocating jpeg buffer",QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
   return;
  }
  mJpegFrame.jpeg_buffer_length=60000;
 }
 //Start draw loop...
 //mDrawActive = true; /* moved into main loop -- DS */
 pthread_create(&mThId,NULL,StartThread,this);
}

void ImageBox::Loop()
{
 int st;
 struct timeval tv1,tv2;
 struct timezone tz;
 long usecs;
 int count,count_frames;

 //Connect to server...
 SetErrorStatus(ERR_NOERRORS,"Connecting...");

 if (mConnection.Connect(mServerName,mServerPort))
 {
  SetErrorStatus(ERR_CRITICAL,"Errors connecting to server");
  mConnection.Disconnect();
  mDrawActive=false;
  return;
 }
 //HTTP request...
 if (mConnection.HttpRequest(HTTP_UA))
 {
  SetErrorStatus(ERR_CRITICAL,"Errors during HTTP request");
  mConnection.Disconnect();
  mDrawActive=false;
  return;
 }
 //parse global headers...
 if (mConnection.ParseGlobalHeaders())
 {
  SetErrorStatus(ERR_CRITICAL,"Errors receiving data");
  mConnection.Disconnect();
  mDrawActive=false;
  return;
 }

 mpMenu->setItemEnabled(connect_id,false);
 mpMenu->setItemEnabled(reconnect_id,false);
 mpMenu->setItemEnabled(disconnect_id,true);
 mpMenu->setItemEnabled(inspect_id,true);

 mpMenu->setItemEnabled(normal_id,true);
 mpMenu->setItemEnabled(stretch_id,true);
 mpMenu->setItemEnabled(fit_id,true);

 SetErrorStatus(ERR_NOERRORS,"Connected");

 gettimeofday(&tv1,&tz);

 mDrawActive = true;

 for(count=0,count_frames=1;mDrawActive;)
 {

  if( (st=mConnection.GetImage(&mJpegFrame)) <=0)
  {
   if(st<0)
    SetErrorStatus(ERR_WARNING,"Errors reading a frame");
   else
   {
    mpMenu->setItemEnabled(reconnect_id,true); //Activate reconnect facility
    if(mConnection.snapshotOnly())
       SetErrorStatus(ERR_WARNING,"Snapshot mode (server busy)"); 
    else
       SetErrorStatus(ERR_WARNING,"Disconnection forced by server"); 
   }
   mConnection.Disconnect();

   //Change menu state...
   mpMenu->setItemEnabled(connect_id,true);
   mpMenu->setItemEnabled(disconnect_id,false);
   mpMenu->setItemEnabled(inspect_id,false);

   mpMenu->setItemEnabled(normal_id,false);
   mpMenu->setItemEnabled(stretch_id,false);
   mpMenu->setItemEnabled(fit_id,false);

   return;
  }
#if ACK
  mConnection.Ack("\r\n",2);
#endif
  pthread_mutex_lock(&mBusy);
  mImage.loadFromData(mJpegFrame.jpeg_buffer,mJpegFrame.length,0);
  pthread_mutex_unlock(&mBusy);

  QTimerEvent *e = new QTimerEvent(EVENT_DRAWFRAME);
  QApplication::postEvent(this,e);

  if ((++count%=count_frames)==0)
  {gettimeofday(&tv2,&tz);
   usecs = (tv2.tv_usec - tv1.tv_usec) + 1000000*(tv2.tv_sec - tv1.tv_sec);
   tv1 = tv2;
   //set next count_frames to emit next message 1 second later
   if (mSelected) emit FrameRateMsg((float)1000000*count_frames/usecs);
   count_frames = (int)((float)1000000*count_frames/usecs)+1;
  }
 }
}

void ImageBox::resizeEvent( QResizeEvent *e )
{
 //Original size... 
 if (mpMenu->isItemChecked(normal_id)) ShowNormal();
 //Stretched...
 if (mpMenu->isItemChecked(stretch_id)) ShowStretch();
 //Fit keeping aspect ratio...
 if (mpMenu->isItemChecked(fit_id)) ShowFit();
 //Send event...
 QTimerEvent *e1 = new QTimerEvent(EVENT_REDRAWFRAME);
 QApplication::postEvent(this,e1);
}

void ImageBox::Disconnect()
{
 SetErrorStatus(ERR_NOERRORS,"Ready");
 sprintf(mStatusMessage,"Disconnecting...");
    
 mDrawActive = false; //shut down loop...
 //Close connection
 if (mConnection.Disconnect())
 {
  SetErrorStatus(ERR_CRITICAL,"Errors disconnecting from server");
 }
 //Wait for draw thread termination...
 if (mThId) pthread_join(mThId,NULL);
 //Change menu state...
 mpMenu->setItemEnabled(connect_id,true);
 mpMenu->setItemEnabled(disconnect_id,false);
 mpMenu->setItemEnabled(inspect_id,false);

 mpMenu->setItemEnabled(normal_id,false);
 mpMenu->setItemEnabled(stretch_id,false);
 mpMenu->setItemEnabled(fit_id,false);

 //Keep previous server name...
 SetErrorStatus(ERR_NOERRORS,"Disconnected.");

 QTimerEvent *e = new QTimerEvent(EVENT_REDRAWFRAME);
 QApplication::postEvent(this,e);
}

void ImageBox::Inspect()
{
 /* TODO */ 
}

void* ImageBox::StartThread(void* data)
{
 ImageBox *pThis = (ImageBox*)data;
 //Start read loop...
 pThis->Loop();
 //Stop thread...
 pthread_exit(0);
}

void ImageBox::timerEvent( QTimerEvent *e )
{
 if (e->timerId() == EVENT_REDRAWFRAME)
 {
  if (!temp_frame.isNull()) 
  {
   mpBox->show();
   bitBlt(mpBox,0,0,&temp_frame);
  }
 }
  
 if (e->timerId() == EVENT_DRAWFRAME)
 {
  //Wait for mImage to be loaded from the buffer...
  pthread_mutex_lock(&mBusy);
  //Resize(if necessary)
  if (mpBox->height()<2 || mpBox->width()<2) ShowNormal();
  //Draw the frame
  if (mDrawActive && !mImage.isNull())
  {//Resize image...
   if ( !mpMenu->isItemChecked(normal_id) )
    temp_frame = mImage.scale(mpBox->size());
   else
    temp_frame = mImage;
   //Paste pixels...
   bitBlt(mpBox,0,0,&temp_frame);
  }
  pthread_mutex_unlock(&mBusy);
 }

 if (e->timerId() == EVENT_REDRAWBACKGROUND)
 {
  mpBox->erase();
  erase();
 }
}

void ImageBox::SetErrorStatus(ErrorType err, char *msg)
{
 mErrorStatus = err;
 strcpy(mStatusMessage,msg);
 switch(mErrorStatus)
 {
  case ERR_WARNING:  if (mSelected) setPaletteBackgroundColor(SEL_WARNING); else setPaletteBackgroundColor(NOTSEL_WARNING);break;
  case ERR_CRITICAL: if (mSelected) setPaletteBackgroundColor(SEL_CRITICAL); else setPaletteBackgroundColor(NOTSEL_CRITICAL);break;
  case ERR_NOERRORS:
  default:           if (mSelected) setPaletteBackgroundColor(SEL_NOERRORS); else setPaletteBackgroundColor(NOTSEL_NOERRORS);break;
 }

 mpBox->hide();
 erase();
 mpBox->show();
 
 if (mSelected) emit StatusMsg(err,msg);
}

char *ImageBox::GetServerName()
{
 return mServerName; 
}

void ImageBox::Select()
{
 mSelected=true;
 switch(mErrorStatus)
 {
  case ERR_WARNING:  if (mSelected) setPaletteBackgroundColor(SEL_WARNING); else setPaletteBackgroundColor(NOTSEL_WARNING);break;
  case ERR_CRITICAL: if (mSelected) setPaletteBackgroundColor(SEL_CRITICAL); else setPaletteBackgroundColor(NOTSEL_CRITICAL);break;
  case ERR_NOERRORS:
  default:           if (mSelected) setPaletteBackgroundColor(SEL_NOERRORS); else setPaletteBackgroundColor(NOTSEL_NOERRORS);break;
                     
 }
 emit StatusMsg(mErrorStatus,mStatusMessage);
 erase();
}

void ImageBox::Deselect()
{
 if (!mSelected) return; //do not waste time redrawing
 mSelected=false;
 switch(mErrorStatus)
 {
  case ERR_WARNING:  if (mSelected) setPaletteBackgroundColor(SEL_WARNING); else setPaletteBackgroundColor(NOTSEL_WARNING);break;
  case ERR_CRITICAL: if (mSelected) setPaletteBackgroundColor(SEL_CRITICAL); else setPaletteBackgroundColor(NOTSEL_CRITICAL);break;
  case ERR_NOERRORS:
  default:           if (mSelected) setPaletteBackgroundColor(SEL_NOERRORS); else setPaletteBackgroundColor(NOTSEL_NOERRORS);break;
 }
 mpBox->hide();
 erase();
 mpBox->show();
 QTimerEvent *e = new QTimerEvent(EVENT_REDRAWFRAME);
 QApplication::postEvent(this,e);
}

void ImageBox::paintEvent(QPaintEvent *)
{

 switch(mErrorStatus)
 {
  case ERR_WARNING:  if (mSelected) setPaletteBackgroundColor(SEL_WARNING); else setPaletteBackgroundColor(NOTSEL_WARNING);break;
  case ERR_CRITICAL: if (mSelected) setPaletteBackgroundColor(SEL_CRITICAL); else setPaletteBackgroundColor(NOTSEL_CRITICAL);break;
  case ERR_NOERRORS:
  default:           if (mSelected) setPaletteBackgroundColor(SEL_NOERRORS); else setPaletteBackgroundColor(NOTSEL_NOERRORS);break;
 }

 if (!mDrawActive && mImage.isNull())
 {
  QTimerEvent *e1 = new QTimerEvent(EVENT_REDRAWBACKGROUND);
  QApplication::postEvent(this,e1);
 }
 QTimerEvent *e2 = new QTimerEvent(EVENT_REDRAWFRAME);
 QApplication::postEvent(this,e2);
}

void ImageBox::AutoConnect(const char *name,const char *port)
{
 strcpy(mServerName,name);
 if (port!=0) strcpy(mServerPort,port); else strcpy(mServerPort,DEFAULT_PORT);
 Reconnect();
}

void ImageBox::AdjustSize()
{
 for (;mImage.isNull();) if (!mDrawActive) return;
 setFixedSize(mImage.width()+1,mImage.height()+2);
 mpBox->resize(mImage.width(),mImage.height());
 setMaximumSize(9999,9999);
}
