/***************************************************************************
                          MainWindow.cpp  -  description
                             -------------------
    begin                : ven mag 21 2004
    copyright            : (C) 2004 by Daniele Lavatelli
    email                : ldanilinux@interfree.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "MainWindow.h"
#include "Servers.h"
#include "SetupDialog.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qiconset.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qmessagebox.h>

static const char rcsid[]="$Id: MainWindow.cpp,v 1.2 2004/07/03 13:20:21 santinod Exp $";

MainWindow::MainWindow(QWidget *parent, const char *name, const char *servername,const char *serverport) : QMainWindow(parent,name)
{
 mpToolbar = new QToolBar(this,"Layout");
 mpToolbar->setHorizontallyStretchable(true);

 mpButtonSetup = new QPushButton(QIconSet(QPixmap("./icons/Connect.bmp",0,QPixmap::Color),QIconSet::Automatic),"Setup",mpToolbar,"buttonsetup");
 mpButtonSetup->setFixedSize(80,30);
 
 QLabel *mpLabelRows = new QLabel("  Rows:",mpToolbar,"labelrows");
 mpLabelRows->setFixedSize(50,20);

 mpSpinRows = new QSpinBox(1,MAX_ROWS,1,mpToolbar,"spinrows");
 mpSpinRows->setFixedSize(50,20);
 mpSpinRows->setValue(1);
 
 QLabel *mpLabelColumns = new QLabel("  Columns:",mpToolbar,"labelrows");
 mpLabelColumns->setFixedSize(80,20);
 
 mpSpinColumns = new QSpinBox(1,MAX_COLUMNS,1,mpToolbar,"spincolumns");
 mpSpinColumns->setFixedSize(50,20);
 mpSpinColumns->setValue(1);
 
 mpToolbar->addSeparator();
 
 mpToolButtonGrid = new QToolButton(mpToolbar,"toolgrid");
 mpToolButtonGrid->setFixedSize(30,30);
 mpToolButtonGrid->setIconSet( QIconSet(QPixmap("./icons/Grid.bmp",0,QPixmap::Color),QIconSet::Automatic) );
 mpToolButtonGrid->setToggleButton(true);

 mpToolButtonInspect = new QToolButton(mpToolbar,"toolinspect");
 mpToolButtonInspect->setFixedSize(30,30);
 mpToolButtonInspect->setIconSet( QIconSet(QPixmap("./icons/Inspect.bmp",0,QPixmap::Color),QIconSet::Automatic) );
 mpToolButtonInspect->setToggleButton(true);
 mpToolButtonGrid->setOn(true);

#ifndef EXPERIMENTAL
 mpToolButtonInspect->setEnabled(false);
 mpToolButtonGrid->setEnabled(false);
#endif

 mpStatusBar = new QStatusBar(this,"statusbar");

 mpStatusIcon = new QLabel(mpStatusBar,"statusicon");
 mpStatusMessage = new QLabel(mpStatusBar,"statusmessage");
 mpStatusServerName = new QLabel(mpStatusBar,"statusservername");
 mpStatusFrameRate = new QLabel(mpStatusBar,"statusframerate");

 mIconNoErrors = QPixmap("./icons/NoErrors.bmp",0,QPixmap::Auto);
 mIconWarning = QPixmap("./icons/Warning.bmp",0,QPixmap::Auto);
 mIconCritical = QPixmap("./icons/Critical.bmp",0,QPixmap::Auto);
 
 mpStatusBar->addWidget(mpStatusIcon,0,false);
 mpStatusBar->addWidget(mpStatusServerName,0,false);
 mpStatusBar->addWidget(mpStatusMessage,0,false);
 mpStatusBar->addWidget(mpStatusFrameRate,0,false);

 mpStatusMessage->setText(" Ready ");
 mpStatusServerName->setText(" Not connected ");
 mpStatusIcon->setPixmap(mIconNoErrors);


 mpCentralWidget = new QWidget(this,"centralwidget");
 setCentralWidget(mpCentralWidget);
 mpMainGrid = new QGridLayout(mpCentralWidget,MAX_ROWS,MAX_COLUMNS,0,1,"grid");

 if (iniOpen()!=0)
 {
  if (iniOpen("./pclient.ini")==0)
  {
   if (QMessageBox::information(this,"Palantir configuration","pclient.ini has been found in the current directory, but the default configuration file, ~/.pclientrc doesn't exist;\nwould you like to copy ./pclient.ini to ~./pclientrc?",QMessageBox::Yes,QMessageBox::No) == QMessageBox::Yes)
    iniSetIgnored(false);
   else
    iniSetIgnored(true);
  }
 }
 else
 {
  if (iniOpen("./pclient.ini")==0)
   QMessageBox::information(this,"Palantir configuration","Found pclient.ini in the current directory;\nwill use it instead of ~./pclientrc.\nNo updates will be made to any of these files.",QMessageBox::Ok);
  iniSetIgnored(true);
 } 

 InitGrid(servername,serverport);

 connect(mpSpinRows,SIGNAL( valueChanged(int) ),this,SLOT( RowsChanged(int) ));
 connect(mpSpinColumns,SIGNAL( valueChanged(int) ),this,SLOT( ColsChanged(int) ));
 connect(mpButtonSetup,SIGNAL( clicked() ),this,SLOT( Config() ));
 
}

inline void MainWindow::InitGrid(const char *servername,const char *serverport)
{
 int r,c;
 for (r=0;r<MAX_ROWS;r++) mpMainGrid->setRowStretch(r,!r);
 for (c=0;c<MAX_COLUMNS;c++) mpMainGrid->setColStretch(c,!c);
 
 for(r=0;r<MAX_ROWS;r++) for(c=0;c<MAX_COLUMNS;c++)
 {mpImageBox[c][r] = new ImageBox(mpCentralWidget,QString("image%1%2").arg(r).arg(c),c,r);
  mpMainGrid->addWidget(mpImageBox[c][r],r,c,0);//AlignCenter);
  connect(mpImageBox[c][r],SIGNAL( Clicked(int,int) ),this,SLOT( SelChanged(int,int) ));
  connect(mpImageBox[c][r],SIGNAL( StatusMsg(ErrorType,char*) ),this,SLOT( GotMsg(ErrorType,char*) ));
  connect(mpImageBox[c][r],SIGNAL( FrameRateMsg(float) ),this,SLOT( GotFrameRateMsg(float) ));
 }
 //select first server
 mSelX = 0;
 mSelY = 0;
 mpImageBox[0][0]->Select();
 //Autoconnect from command line...
 if (servername!=0)
 {
  mpImageBox[0][0]->AutoConnect(servername,serverport);
  mpImageBox[0][0]->AdjustSize();
  return;
 }
 //Autoconnect from ini file...
 if (gDefaultServerAutoconnect)
 {mpImageBox[0][0]->AutoConnect(gServer[gDefaultServer].name,gServer[gDefaultServer].port);
  mpImageBox[0][0]->AdjustSize();
  return;
 }
}

MainWindow::~MainWindow()
{
 //Save server database...
 if (iniClose()!=0)
  QMessageBox::warning(this,"Palantir servers","Errors saving server database",QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
 //delete ImageBoxes...
 for (int r=0;r<MAX_ROWS;r++) for (int c=0;c<MAX_COLUMNS;c++) delete mpImageBox[c][r];
 //delete controls...
 delete mpToolbar;
 delete mpSpinRows;
 delete mpSpinColumns;
#if 0
 delete mpToolButtonGrid;
 delete mpToolButtonInspect;
#endif
 delete mpStatusBar;
 delete mpStatusIcon;
 delete mpStatusMessage;
 delete mpStatusServerName;
 delete mpMainGrid;
 delete mpCentralWidget;
}

void MainWindow::SelChanged(int x, int y) //slot
{
 mSelX = x;
 mSelY = y;
 for (int r=0;r<MAX_ROWS;r++) for (int c=0;c<MAX_COLUMNS;c++)
  if (x==c && y==r) mpImageBox[c][r]->Select(); else mpImageBox[c][r]->Deselect();
 mpStatusServerName->setText(mpImageBox[x][y]->GetServerName());
}

void MainWindow::GotMsg(ErrorType err,char *msg)
{
 mpStatusServerName->setText(mpImageBox[mSelX][mSelY]->GetServerName());
 mpStatusMessage->setText(msg);
 mpStatusFrameRate->setText("");
 switch(err)
 {
  case ERR_NOERRORS : mpStatusIcon->setPixmap(mIconNoErrors);break;
  case ERR_WARNING :  mpStatusIcon->setPixmap(mIconWarning);break;
  case ERR_CRITICAL : mpStatusIcon->setPixmap(mIconCritical);break;
 }
}

void MainWindow::RowsChanged(int new_rows)
{
 mpImageBox[0][0]->setMinimumSize(0,0);
 mpImageBox[0][0]->setMaximumSize(9999,9999);
 
 for (int r=0;r<MAX_ROWS;r++) mpMainGrid->setRowStretch(r,r<new_rows?1:0);
 mpMainGrid->activate();//apply changes
}

void MainWindow::ColsChanged(int new_cols)
{
 mpImageBox[0][0]->setMinimumSize(0,0);
 mpImageBox[0][0]->setMaximumSize(9999,9999);

 for (int c=0;c<MAX_ROWS;c++) mpMainGrid->setColStretch(c,c<new_cols?1:0);
 mpMainGrid->activate();//apply changes
}

void MainWindow::GotFrameRateMsg(float fps)
{char line[10];
 sprintf(line,"%4.1f f/s",fps);
 mpStatusFrameRate->setText(line);  
}

void MainWindow::Config()
{
 SetupDialog *setup_dialog = new SetupDialog(this,"setup",true);
 if (setup_dialog->exec()==QDialog::Rejected) return;
 //Change colors...
 NOTSEL_NOERRORS = setup_dialog->getColor(false,ERR_NOERRORS); 
 SEL_NOERRORS = setup_dialog->getColor(true,ERR_NOERRORS);
 NOTSEL_WARNING = setup_dialog->getColor(false,ERR_WARNING);
 SEL_WARNING = setup_dialog->getColor(true,ERR_WARNING);
 NOTSEL_CRITICAL = setup_dialog->getColor(false,ERR_CRITICAL);
 SEL_CRITICAL = setup_dialog->getColor(true,ERR_CRITICAL);
 //Apply changes...
 repaint();
 iniSetModified();
}
