/***************************************************************************
                          MainWindow.h  -  description
                             -------------------
    begin                : ven mag 21 2004
    copyright            : (C) 2004 by Daniele Lavatelli
    email                : ldanilinux@interfree.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MAIN_WINDOW_H

 #define MAIN_WINDOW_H

 #include <qmainwindow.h>
 #include <qtoolbar.h>
 #include <qspinbox.h>
 #include <qtoolbutton.h>
 #include <qstatusbar.h>
 #include <qpixmap.h>
 #include <qlabel.h>
 #include <qlayout.h>
 #include <qpushbutton.h>
 #include "ImageBox.h"

 #define MAX_ROWS 5
 #define MAX_COLUMNS 5

 ///The application's main window
 class MainWindow : public QMainWindow
 {
  Q_OBJECT

   public:
   ///Creates the main window
   /**\param[in] servername The name of the server (autoconnect mode only)
   \param[in] serverport The port of the server (autoconnect mode only)*/
   MainWindow(QWidget *parent, const char *name, const char *servername = 0,const char *serverport = 0);
   ///Destroys the main window
   ~MainWindow();

   public slots:
   ///Fired by ImageBox when the user changes the selected box in the grid
   /**\param[in] x x coordinate of the new selected box in the grid
   \param[in] y y coordinate of the new selected box in the grid*/
   void SelChanged(int x,int y);
   ///Fired by the active ImageBox when its status message changes
   /**\param[in] err The error status
   \param[in] msg The status message*/
   void GotMsg(ErrorType err,char* msg);
   ///Fired from ImageBox about once a second, when the frame rate is updated
   /**\param[in] fps The frame rate*/
   void GotFrameRateMsg(float fps);

   private slots:
   ///Fired when the user changes the number of rows in the grid, acting on the mpSpinRows spinbox
   /**\param r The new number of rows*/
   void RowsChanged(int r);
   ///Fired when the user changes the number of columns in the grid, acting on the mpSpinColumns spinbox
   /**\param c The new number of columns*/
   void ColsChanged(int c);
   ///Fired when the user presses the "Config" button on the ToolBar
   void Config();
   
   private:
   QToolBar    *mpToolbar;///<Pointer to the main tootBar
   QPushButton *mpConfigButton;
   QSpinBox    *mpSpinRows;///<Pointer to the "Rows" spinbox
   QSpinBox    *mpSpinColumns;///<Pointer to the "Columns" spinbox
   ///Pointer to the "Grid mode" toolbutton
   /**EXPERIMANTAL - not implemented -*/
   QToolButton *mpToolButtonGrid;
   ///Pointer to the "Inspect mode" toolbutton
   /**EXPERIMANTAL - not implemented -*/
   QToolButton *mpToolButtonInspect;
   ///Pointer to the "setup" pushbutton
   QPushButton *mpButtonSetup;
   QStatusBar  *mpStatusBar;///<Pointer to the main statusbar
   QPixmap     mIconNoErrors;///<The "No errors" icon
   QPixmap     mIconWarning;///<The "Warning" icon
   QPixmap     mIconCritical;///<The "Critical error" icon
   QLabel      *mpStatusIcon;///<Pointer to the staus icon on the statusbar
   QLabel      *mpStatusMessage;///<Pointer to the status message on the statusbar
   QLabel      *mpStatusServerName;///<Pointer to the server name on the status bar
   QLabel      *mpStatusFrameRate;///<Pointer to the frame rate box on the status bar
   QGridLayout *mpMainGrid;///<Pointer to the grid
   QWidget     *mpCentralWidget;///<Pointer to the central widget, which contains the grid
   ///Creates the main grid
   /**\param servername Server name (autoconnect mode only)
   \param serverport Server port (autoconnect mode only)*/
   void        InitGrid(const char *servername=0,const char *serverport=0);
   ImageBox    *mpImageBox[MAX_ROWS][MAX_COLUMNS];///<Array of pointers to the ImageBoxes in the grid
   int         mSelX;///<x coordinate of the selected ImageBox in the grid
   int         mSelY;///<y coordinate of the selected ImageBox in the grid
   
 };

#endif
