/***************************************************************************
                          ServerDialog.h  -  description
                             -------------------
    begin                : sab mag 8 2004
    copyright            : (C) 2004 by Daniele Lavatelli
    email                : ldanilinux@interfree.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SERVER_DIALOG_H

 #define SERVER_DIALOG_H

 #include <qdialog.h>
 #include <qpushbutton.h>
 #include <qcombobox.h>
 #include <qlabel.h>
 #include <qpixmap.h>
 #include <qiconset.h>
 #include <stdio.h>
 #include <string.h>
 #include <qmessagebox.h>
 #include <qlineedit.h>
 #include <qcheckbox.h>
 #include "Servers.h"

 ///Lets the user add a new server
 class AddServerDialog : public QDialog
 {
  Q_OBJECT

  public:
  ///Creates the dialog
  AddServerDialog(QWidget *parent, const char *name, bool modal, WFlags f) : QDialog(parent,name,modal,f)
  {//Reset server
   mServer.name[0]='\0';
   mServer.port[0]='\0';
   mServer.label[0]='\0';
   mServer.autoconnect = false;

   setFixedSize(320,210);
   setCaption("New server");

   QLabel *l1 = new QLabel("Type a label for this server:",this,"l1",0);
   l1->setGeometry(10,10,300,20);

   mpTextLabel = new QLineEdit(this,"textlabel");
   mpTextLabel->setGeometry(10,35,300,25);
   mpTextLabel->setPaletteForegroundColor(QColor(0,192,128));
   
   QLabel *l2 = new QLabel("Server address:",this,"l2",0);
   l2->setGeometry(10,70,300,20);

   mpTextName = new QLineEdit(this,"textname");
   mpTextName->setGeometry(10,95,300,25);
   
   QLabel *l3 = new QLabel("Server port:",this,"l3",0);
   l3->setGeometry(10,130,100,25);

   mpTextPort = new QLineEdit(this,"textport");
   mpTextPort->setGeometry(110,130,50,25);
   mpTextPort->setPaletteForegroundColor(QColor(0,0,255));

   mpCheckAutoconnect = new QCheckBox("autoconnect",this,"checkautoconnect");
   mpCheckAutoconnect->setGeometry(170,130,130,25);

   mpButtonAdd = new QPushButton(QIconSet(QPixmap("./icons/Add.bmp",0,QPixmap::Color),QIconSet::Automatic),"Add",this,"buttonadd");
   mpButtonAdd->setGeometry(40,170,100,30);

   mpButtonCancel = new QPushButton(QIconSet(QPixmap("./icons/Cancel.bmp",0,QPixmap::Color),QIconSet::Automatic),"Cancel",this,"buttoncancel");
   mpButtonCancel->setGeometry(160,170,100,30);

   connect(mpButtonAdd,SIGNAL( clicked() ),this,SLOT( accept() ));
   connect(mpButtonCancel,SIGNAL( clicked() ),this,SLOT( reject() ));

   connect(mpTextName,SIGNAL( textChanged(const QString &) ),this,SLOT( NameChanged(const QString &) ));
   connect(mpTextLabel,SIGNAL( textChanged(const QString &) ),this,SLOT( LabelChanged(const QString &) ));
   connect(mpTextPort,SIGNAL( textChanged(const QString &) ),this,SLOT( PortChanged(const QString &) ));

  }

  ///Destroys the dialog
  ~AddServerDialog()
  {//Delete controls...
   delete mpButtonAdd;
   delete mpButtonCancel;
   delete mpTextName;
   delete mpTextPort;
   delete mpTextLabel;
   delete mpCheckAutoconnect; 
  }

  ///Returns the newly added server.
  /**\return The ServerType strycture containing the newly added server*/
  ServerType GetServer()
  {
   return mServer; 
  }
  
  private slots:
  ///This internal slot saves the server name while the user types it in the mpTextName textbox.
  /**\param[in] s A string containing the new server name*/
  void NameChanged(const QString &s)
  {
   strncpy(mServer.name,s.ascii(),245);
  }
  
  ///This internal slot saves the server label while the user types it in the mpTextLabel textbox.
  /**\param[in] s A string containing the new server label*/
  void LabelChanged(const QString &s)
  {
   strncpy(mServer.label,s.ascii(),10);
  }

  ///This internal slot saves the server port while the user types it in the mpTextPort textbox.
  /**\param[in] s A string containing the new server port*/
  void PortChanged(const QString &s)
  {
   strcpy(mServer.port,s.ascii()); 
  }
  
  private:
  ServerType  mServer;///<Structure for the newly added server
  QPushButton *mpButtonAdd;///<pointer to the "Add" button
  QPushButton *mpButtonCancel;///<Pointer to the "Cancel" button
  QLineEdit   *mpTextName;///<Pointer to the "Name" textbox
  QLineEdit   *mpTextPort;///<Pointer to the "Port" textbox
  QLineEdit   *mpTextLabel;///<Pointer to the "Label" textbox
  QCheckBox   *mpCheckAutoconnect;///<Pointer to the "Autoconnect" checkbox
  
 };

 ///Lets the user interact with the server database.
 /**This dialog is mostly used to connect a server, acting on the combobox*/
 class ServerDialog : public QDialog
 {
  Q_OBJECT

  public:
   ///Creates the dialog
   ServerDialog(QWidget *parent, const char *name, bool modal,WFlags f) : QDialog(parent,name,modal,f)
   {
    setFixedSize(380,150);
    setCaption("Connect to server");

    QLabel *l = new QLabel("Select a server from the list, or type a new one:",this,"label");
    l->setGeometry(10,10,300,20);

    mpComboServer = new QComboBox(true,this,"comboserver");
    mpComboServer->setGeometry(10,35,360,25);

    mpButtonAddToList = new QPushButton(QIconSet(QPixmap("./icons/Add.bmp",0,QPixmap::Color),QIconSet::Automatic),"Add server",this,"buttonadd");
    mpButtonAddToList->setGeometry(10,70,110,30);

    mpButtonRemoveFromList = new QPushButton(QIconSet(QPixmap("./icons/Remove.bmp",0,QPixmap::Color),QIconSet::Automatic),"Remove server",this,"buttonremove");
    mpButtonRemoveFromList->setGeometry(125,70,125,30);

    mpButtonSetAsDefault = new QPushButton(QIconSet(QPixmap("./icons/Default.bmp",0,QPixmap::Color),QIconSet::Automatic),"Set as default",this,"buttonsetdefault");
    mpButtonSetAsDefault->setGeometry(255,70,115,30);
    
    mpButtonConnect = new QPushButton(QIconSet(QPixmap("./icons/Connect.bmp",0,QPixmap::Color),QIconSet::Automatic),"Connect",this,"buttonconnect");
    mpButtonConnect->setGeometry(80,110,100,30);

    mpButtonCancel = new QPushButton(QIconSet(QPixmap("./icons/Cancel.bmp",0,QPixmap::Color),QIconSet::Automatic),"Cancel",this,"buttoncancel");
    mpButtonCancel->setGeometry(200,110,100,30);

    mpButtonConnect->setDefault(true);

    connect(mpButtonCancel,SIGNAL( clicked() ),this,SLOT( reject() ));
    connect(mpButtonConnect,SIGNAL( clicked() ),this,SLOT( Connect() ));
    
    connect(mpButtonAddToList,SIGNAL( clicked() ),this,SLOT( AddServer() ));
    connect(mpButtonRemoveFromList,SIGNAL( clicked() ),this,SLOT( RemoveServer() ));
    connect(mpButtonSetAsDefault,SIGNAL( clicked() ),this,SLOT( SetDefaultServer() ));

    RefreshServerList();
   }

   ///Destroys the dialog
   ~ServerDialog()
   {
    delete mpButtonAddToList;
    delete mpButtonConnect;
    delete mpButtonCancel;
    delete mpComboServer;
    delete mpButtonRemoveFromList;
    delete mpButtonSetAsDefault;
   }

   ///Returns the name of the selected server
   /**\return A string containing the server name*/
   char* GetServerName()
   {
    if (mpComboServer->currentItem() == mNewServerIndex) //It is a new server
     return mNewServer.name;
    else
     return gServer[mpComboServer->currentItem()].name; 
   }
   
   ///Returns the port of the selected server
   /**\return A string containing the server port*/
   char* GetServerPort()
   {
    if (mpComboServer->currentItem() == mNewServerIndex) //It is a new server
     if (mNewServer.port[0]=='\0') return DEFAULT_PORT; else return mNewServer.port;
    else
     if (gServer[mpComboServer->currentItem()].port[0]=='\0') return DEFAULT_PORT; else return gServer[mpComboServer->currentItem()].port;
   }

  private:

   ///Updates the combobox's server list
   /**This function reads the server database and adds all its contents to the server list in the combobox mpComboServer
   It also selects the default server*/
   void RefreshServerList()
   {
    int i;
    char line[255];

    mpComboServer->clear();

    for (i=0;i<MAX_SERVERS;i++)
    {
     if (gServer[i].name[0] == '\0') continue;
     if (gServer[i].label[0] != '\0')
      strcpy(line,gServer[i].label);
     else
     {
      strcpy(line,gServer[i].name);
      if (gServer[i].port[0] != '\0')
      {
       strcat(line,":");
       strcat(line,gServer[i].port);
      }
     }
     mpComboServer->insertItem(line);
    }
    //Select default server...
    mpComboServer->setCurrentItem(gDefaultServer);
    //Set the index of a new added server
    mNewServerIndex = mpComboServer->count();
   }

  
  private slots:
   ///Displays the "Add Server" dialog
   void AddServer()
   {
    AddServerDialog *new_server_dialog = new AddServerDialog(this,"newserverdialog",TRUE,0);
    if (new_server_dialog->exec() == QDialog::Accepted)
     if (iniAddServer(new_server_dialog->GetServer())!=0)
      QMessageBox::warning(this,"Add server","Errors writing to server database",QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
    delete new_server_dialog;
    RefreshServerList();
   }

   ///Removes the selected server from the server list
   void RemoveServer()
   {
    if (iniRemoveServer(mpComboServer->currentItem())!=0)
     QMessageBox::warning(this,"Add server","Errors accessing server database",QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
    RefreshServerList();
   }

   ///Marks the selected server as the default server in the server database
   void SetDefaultServer()
   {
    if (iniSetDefaultServer(mpComboServer->currentItem(),gServer[mpComboServer->currentItem()].autoconnect) != 0)
     QMessageBox::warning(this,"Add server","Errors accessing server database",QMessageBox::Ok,QMessageBox::NoButton,QMessageBox::NoButton);
    RefreshServerList();
   }

   ///Hides the dialog, sending the accept() signal to the dialog's parent.
   /**If the selected item is not in the list, this function parses the string, saving server name and port in the mNewServer variable*/
   void Connect()
   {
    int i;
    char *c;
    char line[255];
    
    if (mpComboServer->currentItem() == mNewServerIndex) //It is a new server
    {//Parse text to determine server name and port
     strcpy(line,mpComboServer->currentText());
     //get server name
     c = line;
     for(i=0;c[i]!=':' && c[i]!='\n' && c[i]!=' ' && c[i]!='\r' && i<244;i++) mNewServer.name[i]=c[i];
     mNewServer.name[i] = '\0';
     if (c[i]==':') //there is a port
     {
      //get port
      c = &c[i+1];
      for(i=0;c[i]!='\n' && c[i]!=' ' && c[i]!='\r' && i<9;i++) mNewServer.port[i]=c[i];
      mNewServer.port[i]='\0';
     } else mNewServer.port[0]='\0';
    }
    emit accept();
   }

   
  private:
   QPushButton *mpButtonAddToList;///<Pointer to the "Add" button
   QPushButton *mpButtonConnect;///<Pointer to the "Connect" button
   QPushButton *mpButtonCancel;///<Pointer to the "Cancel" button
   QPushButton *mpButtonRemoveFromList;///<Pointer to the "Remove" button
   QPushButton *mpButtonSetAsDefault;///<Pointer to the "Set default" button
   QComboBox   *mpComboServer;///<Pointer to the "Server" combobox
   int         mNewServerIndex;///<Number of items in the server list (or better, the index of a new item)
   ServerType  mNewServer;///<Used when the user types a new server address which is not contained in the server database
 };

#endif
