/***************************************************************************
                          Servers.cpp  -  description
                             -------------------
    begin                : lun mag 10 2004
    copyright            : (C) 2004 by Daniele Lavatelli
    email                : ldanilinux@interfree.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "Servers.h"
#include "ImageBox.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>


static const char rcsid[]="$Id: Servers.cpp,v 1.1 2004/07/01 19:57:01 santinod Exp $";

ServerType  gServer[MAX_SERVERS];
int         gDefaultServer;
bool        gDefaultServerAutoconnect;
ServerType  gTempServer;

static FILE *fp;
static bool gModified;
static bool gIgnore;
static char gDefaultFilename[256];
static char gIniFile[] = "/.pclientrc";
static char buffer[256];

inline int ini_read()
{
 int s;
 int i;
 char *c;
 char string[512];
 int r,g,b;
 int found_colors=0;

 //Clear server array...
 for(i=0;i<MAX_SERVERS;i++)
 {
  gServer[i].name[0]='\0';
  gServer[i].port[0]='\0';
  gServer[i].label[0]='\0';
  gServer[i].autoconnect = false;
 }

 //search for [global]
 for(;;)
 {
  if (fgets(buffer,256,fp)==NULL)
  {
   perror("ini_read()");
   return -1;
  }
  if (buffer[0] == ';') continue;
  if (strstr(buffer,"[global]") != NULL) break;
 }

 //search for server name and port
 for(;;)
 {
  if (fgets(buffer,256,fp)==NULL)
  {
   perror("ini_read()");
   return -1; 
  }
  if ( (c = strstr(buffer,"defaultserver= ")) != NULL) break;
 }

 //get server name
 c = &c[15];
 for(i=0;c[i]!=':' && c[i]!='\n' && c[i]!=' ' && c[i]!='\r';i++) gTempServer.name[i]=c[i];
 gTempServer.name[i] = '\0';
 if (c[i]==':') //there is a port
 {
  //get port
  c = &c[i+1];
  for(i=0;c[i]!='\n' && c[i]!=' ' && c[i]!='\r';i++) gTempServer.port[i]=c[i];
  gTempServer.port[i]='\0';
 } else gTempServer.port[0]='\0';

 //set label to null
 gTempServer.label[0] = '\0';

 //search for autoconnect status
 for(;;)
 {
  if (fgets(buffer,256,fp)==NULL)
  {perror("ini_read()");
   return -1;
  }
  if ( (c = strstr(buffer,"autoconnect= ")) != NULL) break;
 }
 c=&c[13];
 if (*c == '1') gDefaultServerAutoconnect = true; else gDefaultServerAutoconnect = false;

 //get colors...
 for(;;)
 {
  if (fgets(buffer,256,fp)==NULL)
  {perror("ini_read()");
   return -1;
  }
  if ( (c=strstr(buffer,"sel_noerrors_color= ")) != NULL)
  {
   c=&c[20];
   sscanf(c,"%d %d %d",&r,&g,&b);
   SEL_NOERRORS.setRgb(r,g,b);
   continue;
  }
  if ( (c=strstr(buffer,"notsel_noerrors_color= ")) != NULL)
  {
   c=&c[23];
   sscanf(c,"%d %d %d",&r,&g,&b);
   NOTSEL_NOERRORS.setRgb(r,g,b);
   continue;
  }

  if ( (c=strstr(buffer,"sel_warning_color= ")) != NULL)
  {
   c=&c[19];
   sscanf(c,"%d %d %d",&r,&g,&b);
   SEL_WARNING.setRgb(r,g,b);
   continue;
  }
  if ( (c=strstr(buffer,"notsel_warning_color= ")) != NULL)
  {
   c=&c[22];
   sscanf(c,"%d %d %d",&r,&g,&b);
   NOTSEL_WARNING.setRgb(r,g,b);
   continue;
  }

  if ( (c=strstr(buffer,"notsel_critical_color= ")) != NULL)
  {
   c=&c[23];
   sscanf(c,"%d %d %d",&r,&g,&b);
   NOTSEL_CRITICAL.setRgb(r,g,b);
   continue;
  }
  if ( (c=strstr(buffer,"sel_critical_color= ")) != NULL)
  {
   c=&c[20];
   sscanf(c,"%d %d %d",&r,&g,&b);
   SEL_CRITICAL.setRgb(r,g,b);
   found_colors=1;
   break; //this is the last entry
  }
  if ( strstr(buffer,"[serverlist]")!=NULL ) break;
 }

 //search for [serverlist]
 for(;;)
 {
  if (found_colors) if (fgets(buffer,256,fp)==NULL)
  {
   perror("ini_read()");
   return -1; 
  }
  if (strstr(buffer,"[serverlist]") != NULL) break;
 }

 //search for server name and port
 for(s=0;s<MAX_SERVERS;s++)
 {
  for(;;)
  {if (fgets(buffer,256,fp)==NULL) return 0; //end of file
   sprintf(string,"server%1d= ",s);
   if ( (c = strstr(buffer,string)) != NULL) break;
  }
 
  //get server name
  c = &c[9];
  for(i=0;c[i]!=':' && c[i]!='\n' && c[i]!=' ' && c[i]!=',';i++) gServer[s].name[i]=c[i];
  gServer[s].name[i] = '\0';
  if (c[i]==':') //there is a port
  {
   //get port
   c = &c[i+1];
   for(i=0;c[i]!=',' && c[i]!='\n' && c[i]!=' ' && c[i]!='\r';i++) gServer[s].port[i]=c[i];
   gServer[s].port[i]='\0';
  }
  if (c[i]==',') //there is a label
  {
   //get label
   c = &c[i+2];
   for(i=0;c[i]!='\n' && c[i]!='\r';i++) gServer[s].label[i]=c[i];
   gServer[s].label[i]='\0';
  } else gServer[s].label[0] = '\0';
  //is this the default server?
  if (strcmp(gServer[s].name,gTempServer.name)==0 && strcmp(gServer[s].port,gTempServer.port)==0) gDefaultServer=s;
 }
 return 0;
}

inline int ini_write()
{
 int i;

 clearerr(fp);

 fprintf(fp,"; Sample configuration file for the Windows and Linux (Qt) Palantir clients.\n");
 fprintf(fp,"; The configuration file \"pclient.ini\" must be located in the\n");
 fprintf(fp,"; same directory as the executable.\n");
 fprintf(fp,"; The sections and keys currently recognized are:\n");
 fprintf(fp,";\n");
 fprintf(fp,"; [global]\n");
 fprintf(fp,"; defaultserver= hostname[:port]\n");
 fprintf(fp,"; # server to be displayed in the server address text box\n");
 fprintf(fp,"; # at client startup.\n");
 fprintf(fp,";\n");
 fprintf(fp,"; autoconnect= 0|1\n");
 fprintf(fp,"; # if 1, the client automatically tries to connect\n");
 fprintf(fp,"; # to the default server at startup.\n");
 fprintf(fp,";\n");
 fprintf(fp,"; [not]sel_{noerrors|warning|critical}_color= r g b\n");
 fprintf(fp,"; #specifies the corresponding background color\n");
 fprintf(fp,"; #r, g and b are the red, green and blue components (from 0 to 255)\n");
 fprintf(fp,";\n");
 fprintf(fp,"; [serverlist]\n");
 fprintf(fp,"; server0= hostname[:port] [,description]\n");
 fprintf(fp,"; server1= hostname[:port] [,description]\n");
 fprintf(fp,"; ...\n");
 fprintf(fp,"; server9= hostname[:port] [,description]\n");
 fprintf(fp,"; #  address and (optional) short textual description of the servers\n");
 fprintf(fp,"; #  making up the \"Servers\" menu.\n");
 fprintf(fp,"; #  Servers are listed in the menu by their description, or,\n");
 fprintf(fp,"; #  if none is available, by their address.\n");
 fprintf(fp,"; #  At most 10 servers are allowed (server0...server9).\n");
 
 fprintf(fp,"\n");

 fprintf(fp,"[global]\n");

 fprintf(fp,"defaultserver= %s",gServer[gDefaultServer].name);
 if (gServer[gDefaultServer].port[0] != '\0') fprintf(fp,":%s",gServer[gDefaultServer].port); fprintf(fp,"\n");
 fprintf(fp,"autoconnect= %1d\n",gDefaultServerAutoconnect);

 //Colors...
 fprintf(fp,"notsel_noerrors_color= %d %d %d\n",NOTSEL_NOERRORS.red(),NOTSEL_NOERRORS.green(),NOTSEL_NOERRORS.blue());
 fprintf(fp,"sel_noerrors_color= %d %d %d\n",SEL_NOERRORS.red(),SEL_NOERRORS.green(),SEL_NOERRORS.blue());
 fprintf(fp,"notsel_warning_color= %d %d %d\n",NOTSEL_WARNING.red(),NOTSEL_WARNING.green(),NOTSEL_WARNING.blue());
 fprintf(fp,"sel_warning_color= %d %d %d\n",SEL_WARNING.red(),SEL_WARNING.green(),SEL_WARNING.blue());
 fprintf(fp,"notsel_critical_color= %d %d %d\n",NOTSEL_CRITICAL.red(),NOTSEL_CRITICAL.green(),NOTSEL_CRITICAL.blue());
 fprintf(fp,"sel_critical_color= %d %d %d\n",SEL_CRITICAL.red(),SEL_CRITICAL.green(),SEL_CRITICAL.blue());

 fprintf(fp,"\n");

 fprintf(fp,"[serverlist]\n");
 for (i=0;i<MAX_SERVERS;i++)
 {
  if (gServer[i].name[0] == '\0') continue;
  fprintf(fp,"server%1d= %s",i,gServer[i].name);
  if (gServer[i].port[0] != '\0') fprintf(fp,":%s",gServer[i].port);
  if (gServer[i].label[0] != '\0') fprintf(fp,", %s",gServer[i].label);
  fprintf(fp,"\n");
 }

 return ferror(fp)?-1:0;
}

int iniOpen(char *filename)
{
 int ret;
 fp = fopen(filename,"rt");
 if (fp==NULL)
 {
  perror("ini_open()");
  return -1; 
 }
 ret = ini_read();
 fclose(fp);
 gModified = false;

 return ret;
}

int iniOpen()
{
 int ret;
 
 sprintf(gDefaultFilename,"%s%s",getenv("HOME"),gIniFile);
 
 fp = fopen(gDefaultFilename,"rt");
 if (fp==NULL)
 {
  //perror("ini_open()");
  return -1;
 }
 ret = ini_read();
 fclose(fp);

 gModified = false;

 return ret;
}

int iniClose()
{
 int ret;
 if (gIgnore) 
  return 0;
 //if (!gModified) return 0;
 
 fp = fopen(gDefaultFilename,"wt");
 if (fp==NULL)
 {
  perror("iniClose()");
  return -1;
 }
 ret = ini_write();
 fclose(fp);
 return ret; 
}

int iniAddServer(const ServerType server)
{
 //Add a server in the first empty slot
 int i;
 for(i=0;i<MAX_SERVERS;i++) if (gServer[i].name[0]=='\0')
 {
  strcpy(gServer[i].name,server.name);
  strcpy(gServer[i].port,server.port);
  strcpy(gServer[i].label,server.label);
  gServer[i].autoconnect = server.autoconnect;
  gModified = true;
  return 0;
 }
 return -1;
}

int iniRemoveServer(int n)
{
 int i;
 if (n >= MAX_SERVERS) return -1;
 for(i=n;i<MAX_SERVERS-1;i++) gServer[i] = gServer[i+1];
 gServer[MAX_SERVERS-1].name[0]='\0';
 gServer[MAX_SERVERS-1].port[0]='\0';
 gServer[MAX_SERVERS-1].label[0]='\0';
 gServer[MAX_SERVERS-1].autoconnect = false;
 if (n==gDefaultServer)
 {//Select the first server and set autoconnect to false
  gDefaultServer = 0;
  gDefaultServerAutoconnect = false; 
 }
 gModified = true;
 return 0;
}

int iniSetDefaultServer(int n, bool autoconnect)
{
 if (n>=MAX_SERVERS) return -1;
 gDefaultServer = n;
 gDefaultServerAutoconnect = autoconnect;
 gModified = true;
 return 0;
}

void iniSetModified()
{
 gModified = true; 
}

void iniSetIgnored(bool b)
{
 gIgnore = b;
}
