/***************************************************************************
                          Servers.h  -  description
                             -------------------
    begin                : lun mag 10 2004
    copyright            : (C) 2004 by Daniele Lavatelli
    email                : ldanilinux@interfree.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SERVERS_H

 #define SERVERS_H

 #define MAX_SERVERS 10///<Maximum capacity of the server database
 #define DEFAULT_PORT "3000"///<The deafult port

 ///Contains server data
 struct ServerType
 {
  char name[245];///<Server host
  char port[10];///<Server port
  char label[255];///<Server identifier in the ini file
  bool autoconnect;///<If set to \em true, this server automatically when the client starts
 };

 extern ServerType  gServer[MAX_SERVERS];///<Server database
 extern int         gDefaultServer;///<The index of the default server in the database
 extern bool        gDefaultServerAutoconnect;///<Autoconnect field of the default server

 ///Tries to read the server database from a file and stores its content in the server database
 /**Provided to import ini files from older versions.
 \param[in] filename Name of the ini file.
 \return 0 success \n -1 error: see the stderr output for details.*/
 int iniOpen(char *filename);
 ///Overloaded. Acts as the above function, searching for the file "pclient.ini"
 /**\return 0 success \n -1 error: see the stderr output for details.*/
 int iniOpen();
 ///Saves the server database into the default ini file "pclient.ini"
 /** \return 0 success \n -1 error: see the stderr output for details.*/
 int iniClose();
 ///Adds a server to the database
 /**The iniOpen() function must be called before using this function.
 \param[in] server The ServerType structure of the new server
 \return 0 success \n -1 error: server database full.*/
 int iniAddServer(const ServerType server);
 ///Removes the n-th server from the database
 /**The iniOpen() function must be called before using this function.
 \param[in] n The index of the server which must be removed
 \return 0 success \n -1 error: \em n index out of range*/
 int iniRemoveServer(int n);
 ///Sets the n-th server as the default server
 /**The iniOpen() function must be called before using this function.
 \param[in] n The index of the new default server
 \param[in] autoconnect If \em true, the new default server connects automatically when pclient starts
 \return 0 success \n -1 error: \em n index out of range*/
 int iniSetDefaultServer(int n, bool autoconnect);
 ///Forces a rewriting of the ini file when the application terminates
 void iniSetModified();
 ///If \em true is passed, does not rewrites the default ini file at shutdown.
 void iniSetIgnored(bool b);
#endif
