/***************************************************************************
                          SetupDialog.h  -  description
                             -------------------
    begin                : dom nov 14 2004
    copyright            : (C) 2004 by Daniele Lavatelli
    email                : ldaniele@interfree.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SETUP_DIALOG_H

 #define SETUP_DIALOG_H

 #include <qpushbutton.h>
 
 #include <qdialog.h>
 #include <qcolor.h>
 #include <qwidget.h>
 #include <qgroupbox.h>
 #include <qcolordialog.h>
 #include "ImageBox.h"

 class SetupDialog: public QDialog
 {
  Q_OBJECT

  public:
   SetupDialog(QWidget * parent = 0, const char * name = 0, bool modal = FALSE, WFlags f = 0)
   {

    setFixedSize(260,220);

    setCaption("pclient setup");
      
    mpColorBox = new QGroupBox("Colors",this,"ColorGroup");
    mpColorBox->setGeometry(5,5,250,170);

    mpNotselLabel = new QLabel("Unselected",mpColorBox,"notsellabel");
    mpNotselLabel->setGeometry(10,20,70,20);
    
    mpSelLabel = new QLabel("Selected",mpColorBox,"sellabel");
    mpSelLabel->setGeometry(100,20,70,20);

    mpNotselNoerrorsBox = new QPushButton(mpColorBox,"notselnoerrorsbox");
    mpNotselNoerrorsBox->setGeometry(20,40,50,30);
    mpNotselNoerrorsBox->setPaletteBackgroundColor(NOTSEL_NOERRORS);
        
    mpSelNoerrorsBox = new QPushButton(mpColorBox,"selnoerrorsbox");
    mpSelNoerrorsBox->setGeometry(105,40,50,30);
    mpSelNoerrorsBox->setPaletteBackgroundColor(SEL_NOERRORS);

    mpNoerrorsLabel = new QLabel("No Errors",mpColorBox,"noerrorslabel");
    mpNoerrorsLabel->setGeometry(180,45,55,20);

    mpNotselWarningBox = new QPushButton(mpColorBox,"notselwarningbox");
    mpNotselWarningBox->setGeometry(20,80,50,30);
    mpNotselWarningBox->setPaletteBackgroundColor(NOTSEL_WARNING);

    mpSelWarningBox = new QPushButton(mpColorBox,"selwarningbox");
    mpSelWarningBox->setGeometry(105,80,50,30);
    mpSelWarningBox->setPaletteBackgroundColor(SEL_WARNING);

    mpWarningLabel = new QLabel("Warning",mpColorBox,"warninglabel");
    mpWarningLabel->setGeometry(180,85,55,20);

    mpNotselCriticalBox = new QPushButton(mpColorBox,"notselcriticalbox");
    mpNotselCriticalBox->setGeometry(20,120,50,30);
    mpNotselCriticalBox->setPaletteBackgroundColor(NOTSEL_CRITICAL);

    mpSelCriticalBox = new QPushButton(mpColorBox,"selcriticalbox");
    mpSelCriticalBox->setGeometry(105,120,50,30);
    mpSelCriticalBox->setPaletteBackgroundColor(SEL_CRITICAL);

    mpCriticalLabel = new QLabel("Critical",mpColorBox,"criticallabel");
    mpCriticalLabel->setGeometry(180,125,55,20);

    mpOk = new QPushButton(QIconSet(QPixmap("./icons/NoErrors.bmp",0,QPixmap::Color),QIconSet::Automatic),"OK",this,"okbutton");
    mpOk->setGeometry(10,180,80,30);

    mpCancel = new QPushButton(QIconSet(QPixmap("./icons/Cancel.bmp",0,QPixmap::Color),QIconSet::Automatic),"Cancel",this,"cancelbutton");
    mpCancel->setGeometry(100,180,150,30);

    connect(mpNotselNoerrorsBox,SIGNAL( clicked() ),this,SLOT( NotselNoerrorsClicked() ));
    connect(mpSelNoerrorsBox,SIGNAL( clicked() ),this,SLOT( SelNoerrorsClicked() ));
    connect(mpNotselWarningBox,SIGNAL( clicked() ),this,SLOT( NotselWarningClicked() ));
    connect(mpSelWarningBox,SIGNAL( clicked() ),this,SLOT( SelWarningClicked() ));
    connect(mpNotselCriticalBox,SIGNAL( clicked() ),this,SLOT( NotselCriticalClicked() ));
    connect(mpSelCriticalBox,SIGNAL( clicked() ),this,SLOT( SelCriticalClicked() ));
    connect(mpOk,SIGNAL( clicked() ),this,SLOT( accept() ));
    connect(mpCancel,SIGNAL( clicked() ),this,SLOT( reject() ));
    
   }
   
   ~SetupDialog()
   {
    delete mpNotselNoerrorsBox;
    delete mpNotselWarningBox;
    delete mpNotselCriticalBox;
    delete mpSelNoerrorsBox;
    delete mpSelWarningBox;
    delete mpSelCriticalBox;
    delete mpColorBox;
    delete mpNotselLabel;
    delete mpSelLabel;
    delete mpNoerrorsLabel;
    delete mpWarningLabel;
    delete mpCriticalLabel;
    delete mpOk;
    delete mpCancel;
   }
   
   QColor getColor(int selected, ErrorType err)
   {
    if (selected && err==ERR_NOERRORS) return mpSelNoerrorsBox->paletteBackgroundColor();
    if (!selected && err==ERR_NOERRORS) return mpNotselNoerrorsBox->paletteBackgroundColor();
    if (selected && err==ERR_WARNING) return mpSelWarningBox->paletteBackgroundColor();
    if (!selected && err==ERR_WARNING) return mpNotselWarningBox->paletteBackgroundColor();
    if (selected && err==ERR_CRITICAL) return mpSelCriticalBox->paletteBackgroundColor();
    if (!selected && err==ERR_CRITICAL) return mpNotselCriticalBox->paletteBackgroundColor();
    return QColor(0,0,255);
   }
   
   void   setColor(int selected, ErrorType err, char r, char g, char b)
   {
    if (selected && err==ERR_NOERRORS) mpSelNoerrorsBox->setPaletteBackgroundColor(QColor(r,g,b));
    if (!selected && err==ERR_NOERRORS) mpNotselNoerrorsBox->setPaletteBackgroundColor(QColor(r,g,b));
    if (selected && err==ERR_WARNING) mpSelWarningBox->setPaletteBackgroundColor(QColor(r,g,b));
    if (!selected && err==ERR_WARNING) mpNotselWarningBox->setPaletteBackgroundColor(QColor(r,g,b));
    if (selected && err==ERR_CRITICAL) mpSelCriticalBox->setPaletteBackgroundColor(QColor(r,g,b));
    if (!selected && err==ERR_CRITICAL) mpNotselCriticalBox->setPaletteBackgroundColor(QColor(r,g,b));
   }

  private:
   QPushButton *mpNotselNoerrorsBox;
   QPushButton *mpNotselWarningBox;
   QPushButton *mpNotselCriticalBox;
   QPushButton *mpSelNoerrorsBox;
   QPushButton *mpSelWarningBox;
   QPushButton *mpSelCriticalBox;
   QPushButton *mpOk;
   QPushButton *mpCancel;
   QGroupBox   *mpColorBox;
   QLabel      *mpNotselLabel;
   QLabel      *mpSelLabel;
   QLabel      *mpNoerrorsLabel;
   QLabel      *mpWarningLabel;
   QLabel      *mpCriticalLabel;

  private slots:
   void NotselNoerrorsClicked()
   {
    QColor c = QColorDialog::getColor(NOTSEL_NOERRORS,this,"colordialog");
    if (!c.isValid()) return;
    mpNotselNoerrorsBox->setPaletteBackgroundColor(c);
   }
   void NotselWarningClicked()
   {
    QColor c = QColorDialog::getColor(NOTSEL_WARNING,this,"colordialog");
    if (!c.isValid()) return;
    mpNotselWarningBox->setPaletteBackgroundColor(c);
   }
   void NotselCriticalClicked()
   {
    QColor c = QColorDialog::getColor(NOTSEL_CRITICAL,this,"colordialog");
    if (!c.isValid()) return;
    mpNotselCriticalBox->setPaletteBackgroundColor(c);
   }
   void SelNoerrorsClicked()
   {
    QColor c = QColorDialog::getColor(SEL_NOERRORS,this,"colordialog");
    if (!c.isValid());
    mpSelNoerrorsBox->setPaletteBackgroundColor(c);
   }
   void SelWarningClicked()
   {
    QColor c = QColorDialog::getColor(SEL_WARNING,this,"colordialog");
    if (!c.isValid());
    mpSelWarningBox->setPaletteBackgroundColor(c);
   }
   void SelCriticalClicked()
   {
    QColor c = QColorDialog::getColor(SEL_CRITICAL,this,"colordialog");
    if (!c.isValid()) return;
    mpSelCriticalBox->setPaletteBackgroundColor(c);
   }
 };

#endif
