
/*
    Logging utilities.

    Copyright 2000-2007 by David Santinoli <david@santinoli.com>
*/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include <sys/time.h>
#include <pthread.h>
#include "diag.h"
#include "parser.h"

static const char rcsid[]="$Id: diag.c,v 1.7 2007/05/18 23:14:40 santinod Exp $";

static pthread_mutex_t diag_mtx=PTHREAD_MUTEX_INITIALIZER;
static FILE *lf;
int debug_level;


/* close and reopen log file */
void diag_restart(char *log_file)
{
 pthread_mutex_lock(&diag_mtx);
 fclose(lf);
 lf=fopen(log_file,"a");
 pthread_mutex_unlock(&diag_mtx);
 if(!lf) {
    lf=stderr;
    diag(ERR,"Cannot reopen log file `%s'",log_file);
    }
 diag(INFO,"Log restarted\n");
}


/* set log file. Invoked first with log_file==0 (config file parse error
   are logged to stderr), then - if a LogFile directive was found - with the
   proper file name */
void diag_setfile(char *log_file)
{
 FILE *f;

 lf=stderr;
 if(!log_file) return;
 if(!(f=fopen(log_file,"a"))) diag(ERR,"Cannot open log file `%s'",log_file);
 else lf=f;
}


void diag(int pri, char *format, ...)
{
 va_list ap;
 static char *subsystem[]={"","main","net","audio","devices","config","http",
    "video"};
 int subsys=pri>>9;

 if((pri&(ERRMASK|DIE))<debug_level) return;
 va_start(ap, format);
 pthread_mutex_lock(&diag_mtx);
 if(!(pri&NOTIME)) {
#if 1
    char timestamp[60];
    time_t now;
    time(&now);
    if(strftime(timestamp,60,"%b %e %T",localtime(&now) ) )
       fprintf(lf,"%s ",timestamp);
#else
    struct timeval tv;
    gettimeofday(&tv,0);
    fprintf(lf,"%ld.%06ld ",tv.tv_sec,tv.tv_usec);
#endif
    }
 if(subsys) fprintf(lf,"[%s] ",subsystem[subsys]);
 if(pri&DIE) fprintf(lf,"** Fatal error: ");
 vfprintf(lf,format,ap);
 if(pri&ERRNO) fprintf(lf,": %s\n",strerror(errno));
 if(lf!=stderr) fflush(lf);
 pthread_mutex_unlock(&diag_mtx);
 va_end(ap);
 if(pri&DIE) {
    if(lf!=stderr) fclose(lf);
    exit(pri);
    }
}
