
/*
    JPEG output functions.

    Copyright 2000-2007 by David Santinoli <david@santinoli.com>
*/

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <jpeglib.h>
#include "main.h"
#include "diag.h"
#include "parser.h"

static JSAMPROW* rows;
static struct jpeg_compress_struct jcomp;
static struct jpeg_error_mgr jerr;
static struct jpeg_destination_mgr dstmgr;
unsigned char *jpeg_buf;
unsigned long int jpeg_bytes;
unsigned long int jpeg_buf_size=32000;


static const char rcsid[]="$Id: jpeg.c,v 1.13 2007/05/18 23:18:01 santinod Exp $";

extern pthread_mutex_t jpeg_buf_mtx;
extern pthread_mutex_t jpeg_buf_size_mtx;
extern int current_frame;

static void jpeg_init_destination (struct jpeg_compress_struct *cinfo)
{
 cinfo->dest->next_output_byte=jpeg_buf;
 cinfo->dest->free_in_buffer=jpeg_buf_size;
}


static int jpeg_empty_output_buffer(struct jpeg_compress_struct *cinfo)
{
 if( (jpeg_buf=realloc(jpeg_buf,jpeg_buf_size*2))==0 ) {
    diag(SS_VIDEO|ERROR|DIE,"Failed to resize JPEG compression buffer to %lu\n",jpeg_buf_size*2);
    }
 diag(SS_VIDEO|DEBUG,"JPEG compression buffer resized to %lu\n",jpeg_buf_size*2);
 cinfo->dest->next_output_byte=jpeg_buf+jpeg_buf_size;
 cinfo->dest->free_in_buffer=jpeg_buf_size;
 pthread_mutex_lock(&jpeg_buf_size_mtx);
 jpeg_buf_size*=2;
 pthread_mutex_unlock(&jpeg_buf_size_mtx);
 return 1;
}


static void jpeg_term_destination(struct jpeg_compress_struct *cinfo)
{
#if 0
 FILE *outfile;
 if((outfile=fopen("out.jpg","w"))==NULL)
    diag(SS_VIDEO|DIERR,"jpeg: can't open file");
 fwrite(jpeg_buf,jpeg_buf_size-cinfo->dest->free_in_buffer,1,outfile);
 fclose(outfile);
#endif
}


void jpeg_init(void)
{
 if((jpeg_buf=(unsigned char*)malloc(jpeg_buf_size))==0)
    diag(SS_VIDEO|DIE,"failed malloc()ing JPEG compression buffer\n");
 jcomp.err=jpeg_std_error(&jerr);

 /* Allocate and initialize a JPEG compression object */
 jpeg_create_compress(&jcomp);
 jcomp.dct_method=JDCT_FASTEST;
 dstmgr.init_destination=jpeg_init_destination;
 dstmgr.empty_output_buffer=jpeg_empty_output_buffer;
 dstmgr.term_destination=jpeg_term_destination;
 /* Specify the destination for the compressed data */
 jcomp.dest=&dstmgr;
}


void jpeg_compress_init(int x, int y, unsigned char *base, int mono, int quality)
{
 int i;

 rows=(JSAMPROW*)malloc(y*sizeof(JSAMPROW));
 for(i=0;i<y;i++)
    if(config.image_filters&IMAGE_FILTER_INVERT)
       rows[y-i-1]=(mono ? (JSAMPROW)(base+x*i): (JSAMPROW)(base+3*x*i));
    else
       rows[i]=(mono ? (JSAMPROW)(base+x*i): (JSAMPROW)(base+3*x*i));

 /* Set parameters for compression, including image size & colorspace */
 jcomp.image_width=x;
 jcomp.image_height=y;
 jcomp.input_components=(mono?1:3);
 jcomp.in_color_space=(mono?JCS_GRAYSCALE:JCS_RGB);
 /* Note: in case VIDEO_PALETTE_YUV420P is used, setting
    jcomp.in_color_space=JCS_YCbCr is of no help, since libjpeg does not
    handle YCbCr => RGB conversion */
 jpeg_set_defaults(&jcomp);
 jpeg_set_quality(&jcomp,quality,TRUE);
}


void jpeg_free_rows(void)
{
 free(rows);
}


void jpeg_cleanup(void)
{
 /* Release the JPEG compression object */
 jpeg_destroy_compress(&jcomp);
 free(jpeg_buf);
}


void jpeg_compress(int x, int y)
{
 int n=0;
 jpeg_start_compress(&jcomp,TRUE);
 pthread_mutex_lock(&jpeg_buf_mtx);
 while(n<y) {
    n+=jpeg_write_scanlines(&jcomp,rows+n,y-n);
    }
 jpeg_finish_compress(&jcomp);
 jpeg_bytes=jcomp.dest->next_output_byte-jpeg_buf;
 current_frame++;
 pthread_cond_broadcast(&new_frame_cond);
 pthread_mutex_unlock(&jpeg_buf_mtx);
}


void jpeg_load_file(char *filename)
{
 int f;
 unsigned long int n;

 if((f=open(filename,O_RDONLY))==-1)
    diag(SS_VIDEO|DIERR,"Error opening file `%s'",filename);
 jpeg_bytes=0;

 if((n=read(f,jpeg_buf,jpeg_buf_size))==-1)
    diag(SS_VIDEO|DIERR,"Error reading file `%s'",filename);
 jpeg_bytes=n;
 diag(SS_VIDEO|DEBUG,"Loaded file `%s' (%d bytes)\n",filename,jpeg_bytes);
}
